﻿%%FRAGMENTINPUT_STRUCT%%

%%FRAGMENTOUTPUT_STRUCT%%

%%SAMPLERFRONT_BINDING%% var samplerFront : sampler;
%%TEXTUREFRONT_BINDING%% var textureFront : texture_2d<f32>;

//%//%SAMPLERBACK_BINDING%//% var samplerBack : sampler;
//%//%TEXTUREBACK_BINDING%//% var textureBack : texture_2d<f32>;

//%//%SAMPLERDEPTH_BINDING%//% var samplerDepth : sampler;
//%//%TEXTUREDEPTH_BINDING%//% var textureDepth : texture_depth_2d;

struct ShaderParams {
	u_glitchStrength: f32,
	u_random1: f32,
	u_random2: f32,
	u_random3: f32,
	u_random4: f32
};

%%SHADERPARAMS_BINDING%% var<uniform> shaderParams : ShaderParams;

%%C3PARAMS_STRUCT%%

%%C3_UTILITY_FUNCTIONS%%

fn sat(t: f32) -> f32 {
	return clamp(t, 0., 1.);
} 

fn remap(t: f32, a: f32, b: f32) -> f32 {
	return sat((t - a) / (b - a));
} 

fn linterp(t: f32) -> f32 {
	return sat(1. - abs(2. * t - 1.));
} 

fn spectrum_offset(t: f32) -> vec3<f32> {
	var ret: vec3<f32>;
	let lo: f32 = step(t, 0.5);
	let hi: f32 = 1. - lo;
	let w: f32 = linterp(remap(t, 1. / 6., 5. / 6.));
	let neg_w: f32 = 1. - w;
	ret = vec3<f32>(lo, 1., hi) * vec3<f32>(neg_w, w, neg_w);
	return pow(ret, vec3<f32>(1. / 2.2));
} 

fn rand(n: vec2<f32>) -> f32 {
	return fract(sin(dot(n.xy, vec2<f32>(12.9898, 78.233))) * 43758.547);
} 

fn srand(n: vec2<f32>) -> f32 {
	return rand(n) * 2. - 1.;
} 

fn mytrunc2(x: vec2<f32>, num_levels: f32) -> vec2<f32> {
	return floor(x * num_levels) / num_levels;
} 

fn mytrunc(x: f32, num_levels: f32) -> f32 {
	return floor(x * num_levels) / num_levels;
}

@fragment
fn main(input : FragmentInput) -> FragmentOutput
{
	var uv: vec2<f32> = input.fragUV;
	uv.y = uv.y;
	var time: f32 = ((c3Params.seconds * 100.) % (32.)) / 110.;
	let GLITCH: f32 = shaderParams.u_glitchStrength / 10.;
	let gnm: f32 = sat(GLITCH);
	let rnd0: f32 = rand(mytrunc2(vec2<f32>(time, time), 6.)) * shaderParams.u_random1;
	let r0: f32 = sat((1. - gnm) * 0.7 + rnd0);
	let rnd1: f32 = rand(vec2<f32>(mytrunc(uv.x, 10. * r0), time)) * shaderParams.u_random2;
	var r1: f32 = 0.5 - 0.5 * gnm + rnd1;
	r1 = 1. - max(0., select( r1, 0.9999999, r1 < 1.));
	let rnd2: f32 = rand(vec2<f32>(mytrunc(uv.y, 40. * r1), time)) * shaderParams.u_random3;
	let r2: f32 = sat(rnd2);
	let rnd3: f32 = rand(vec2<f32>(mytrunc(uv.y, 10. * r0), time)) * shaderParams.u_random4;
	let r3: f32 = 1. - sat(rnd3 + 0.8) - 0.1;
	let pxrnd: f32 = rand(uv + time);
	var ofs: f32 = 0.05 * r2 * GLITCH * (select(1., -1., rnd0 > 0.5 ));
	ofs = ofs + (0.5 * pxrnd * ofs);
	uv.y = uv.y + (0.1 * r3 * GLITCH);
	var NUM_SAMPLES: i32 = 20;
	let RCP_NUM_SAMPLES_F: f32 = 1. / f32(NUM_SAMPLES);
	var sum: vec4<f32> = vec4<f32>(0.);
	var wsum: vec3<f32> = vec3<f32>(0.);

	for (var i: i32 = 0; i < NUM_SAMPLES; i = i + 1) {
		let t: f32 = f32(i) * RCP_NUM_SAMPLES_F;
		uv.x = sat(uv.x + ofs * t);
		var samplecol: vec4<f32> = textureSampleBias(textureFront, samplerFront, uv, -10.);
		let s: vec3<f32> = spectrum_offset(t);
		var samplecolrgb = samplecol.rgb;
		samplecolrgb = samplecol.rgb * s;
		samplecol.r = samplecolrgb.x;
		samplecol.g = samplecolrgb.y;
		samplecol.b = samplecolrgb.z;
		sum = sum + (samplecol);
		wsum = wsum + (s);
	}

	var sumrgb = sum.rgb;
	sumrgb = sum.rgb / (wsum);
	sum.r = sumrgb.x;
	sum.g = sumrgb.y;
	sum.b = sumrgb.z;
	sum.a = sum.a * (RCP_NUM_SAMPLES_F);

	var output : FragmentOutput;
	output.color = sum;
	return output;
} 
